<?php //$Id: block_multimovie.php,v 1.8 2007/09/23 17:37:07 stronk7 Exp $

///////////////////////////////////////////////////////////////////////////
//                                                                       //
// NOTICE OF COPYRIGHT                                                   //
//                                                                       //
// Moodle - Modular Object-Oriented Dynamic Learning Environment         //
//          http://moodle.com                                            //
//                                                                       //
// Copyright (C) 2001-3001 Martin Dougiamas        http://dougiamas.com  //
//           (C) 2001-3001 Eloy Lafuente (stronk7) http://contiento.com  //
//                                                                       //
// This program is free software; you can redistribute it and/or modify  //
// it under the terms of the GNU General Public License as published by  //
// the Free Software Foundation; either version 2 of the License, or     //
// (at your option) any later version.                                   //
//                                                                       //
// This program is distributed in the hope that it will be useful,       //
// but WITHOUT ANY WARRANTY; without even the implied warranty of        //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         //
// GNU General Public License for more details:                          //
//                                                                       //
//          http://www.gnu.org/copyleft/gpl.html                         //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

// This block allows you to display videos from different sources (youtube,
// google...) inside one Moodle block.
//
// Syntax for videos is:
//     [[mm:source:reference|title]]
// where:
//     mm:        acronym of "MultiMovie", must be always present.
//     source:    one of these are supported: "youtube", "google" "teachertube"
//                (without the quotes!).
//     reference: reference of the video to display (can be easily copied
//                from the URL when displaying the video on the source site)
//     title:     free text to be displayed in the popup that will be
//                generated if more than one video is introduced.

class block_multimovie extends block_base {

    function init() {
        $this->title = get_string('multimovie', 'block_multimovie');
        $this->version = 2006121900;
    }

    function preferred_width() {
        return 180;
    }

    function applicable_formats() {
        return array('all' => true);
    }

    function instance_allow_multiple() {
        return true;
    }

    function specialization() {
        $this->title = isset($this->config->title) ? $this->config->title : get_string('newmultimovieblock', 'block_multimovie');
    }

    function get_content() {

    /// Perhaps content has been defined previously in this request
        if ($this->content !== NULL) {
            return $this->content;
        }

    /// Initialize content
        $this->content = new stdClass;

    /// Get the list of movies defined in the block
        $movies = $this->get_movies();

    /// No movies, unconfigured
        if (empty($movies)) {
            $this->content->text = get_string('unconfiguredblock', 'block_multimovie');
            $this->content->footer = '';
            return $this->content;
        }

    /// Get the popup menu and display it in the block footer
        $this->content->footer = $this->get_movies_popup($movies);

    /// The content in one div
        $this->content->text = '<div id="blockmultimovie-' . $this->instance->id . '" style="text-align: center">';

    /// Based in config->type, process the contents...
        if ($this->first->type == 'youtube') {
            $this->content->text.= $this->first_youtube();
        } else if ($this->first->type == 'google') {
            $this->content->text.= $this->first_google();
        } else if ($this->first->type == 'teachertube') {
            $this->content->text.= $this->first_teachertube();
        }
        /// And add the custom javascript to handle this menu
            $this->content->footer .= $this->get_js();

    /// Close the content div
        $this->content->text.= '</div>';

        return $this->content;
    }

    /**
     * This function will return the array of movies defined in the block
     */
    function get_movies() {

        global $CFG;

    /// No movies if empty
        if (empty($this->config->text)) {
            return false;
        }

    /// Initial values
        $text = $this->config->text;
        $movies = array(); /// Here we'll store all the valid movies
        $u = empty($CFG->unicodedb) ? '' : 'u'; //Unicode modifier

    /// Process config text to find references (new behaviour, mediawiki-like syntax, supporting
    /// different video providers: youtube, google)
    /// Syntax for each reference: [[mm:type:reference|title]]
        preg_match_all('/\[\[mm:(youtube|google|teachertube):(.*?)\|(.*?)\]\]/s'.$u, $text, $list_of_movies);

    /// If we have found references, let's process them
        if (!empty($list_of_movies[0])) {
            foreach ($list_of_movies[0] as $key=>$item) {
                $movie = new stdClass;
                $movie->type = $list_of_movies[1][$key];
                $movie->reference = $list_of_movies[2][$key];
                $movie->title = $list_of_movies[3][$key];
                $movies[] = $movie;
            }
        } else {
        /// Process config text to find references (old behaviour, individual lines only supporting youtube)
        /// Syntax for each reference: One line with this format-> reference|title
            $text = str_replace("\r", "\n", str_replace("\n\r", "\r", $text)); /// Normalize linefeeds
            $lines = explode("\n", $text); /// Split text in lines
            if ($lines) {
                foreach($lines as $line) {
                    $parts = explode("|", $line);
                    if (isset($parts[0]) && isset($parts[1])) {
                        $movie = new stdClass;
                        $movie->type = 'youtube';
                        $movie->reference = $parts[0];
                        $movie->title = $parts[1];
                        $movies[] = $movie;
                    }
                }
            }
        }

    /// Finally, return the array of movies
        return $movies;
    }

    /**
     * This function will return the popup menu of movies defined in the block
     */
    function get_movies_popup($movies) {

    /// Check we have movies
        if (empty($movies)) {
            return NULL;
        }

    /// This variable will contain the popup html code
        $result = '';

    /// Save the first movie of the list, we'll need it later
        reset($movies);
        $this->first = $movies[0];

    /// We have found valid movies, build the menu
    /// If it has more than 1 element, draw the menu
        if (count($movies) > 1) {
        /// Convert movies to an array of links to be used in the menu
            $links = array();
            foreach ($movies as $movie) {
                $links[$movie->type.$movie->reference] = format_string($movie->title);
            }

        /// Build the popup menu based in the links
            $result = choose_from_menu($links,
                          'blockmultimovie-menu-'.$this->instance->id,
                          $this->first->type.$this->first->reference,
                          '',
                          'blockmultirmovierefresh'.$this->instance->id.'(this.value)',
                          0,
                          true);
        }
    /// Return the popup html output (or empty if only 1 video is available)
        return $result;
    }

    /**
     * This function will generate the html output to preview the first element of one
     * youtube content initially
     */
    function first_youtube() {

        return '<div id="blockmultimovie-container-' . $this->instance->id . '">'.
               '<object type="application/x-shockwave-flash" '.
               'data="http://www.youtube.com/v/'.$this->first->reference.'" '.
               'width="170" height="140">'.
               '<param name="movie" value="http://www.youtube.com/v/'.$this->first->reference.'" />'.
               '<param name="wmode" value="transparent" />'.
               '</object></div>';
    }

    /**
     * This function will generate the html output to preview the first element of one
     * google content initially
     */
    function first_google() {

        return '<div id="blockmultimovie-container-' . $this->instance->id . '">'.
               '<object type="application/x-shockwave-flash" '.
               'data="http://video.google.com/googleplayer.swf?docId='.$this->first->reference.'&amp;hl=en&amp;playerMode=simple" '.
               'width="170" height="140">'.
               '<param name="movie" value="http://video.google.com/googleplayer.swf?docId='.$this->first->reference.'&amp;hl=en&amp;playerMode=simple" />'.
               '<param name="wmode" value="transparent" />'.
               '</object></div>';
    }

    /**
     * This function will generate the html output to preview the first element of one
     * teachertube content initially
     */
    function first_teachertube() {

        return '<div id="blocmovie-container-' . $this->instance->id . '">'.
               '<object type="application/x-shockwave-flash" '.
               'data="http://www.teachertube.com/flvplayer.swf" '.
               'width="170" height="140" >'.
               '<param name="FlashVars" value="config=http://www.teachertube.com/flvplayer.php?viewkey='.$this->first->reference.
                                              '&amp;vimg=http://www.teachertube.com/thumb/10.jpg" />'.
               '<param name="wmode" value="transparent" />'.
               '</object></div>';
    }

    /**
     * This function will generate the js output needed to modify the
     * video contents directly
     */
    function get_js() {

    /// script header
        $result = '<script type="text/javascript" defer="defer">'."\n";
        $result.= '//<![CDATA['."\n";
        $result.= '    function blockmultirmovierefresh'.$this->instance->id.'(value) {'."\n";
        $result.= "        var type;"."\n";
        $result.= "        var reference;"."\n";
        $result.= "        var txt;"."\n";
    /// based on value, calculate the type
        $result.= "        if (value.indexOf('youtube') != -1) {"."\n";
        $result.= "            type='youtube';"."\n";
        $result.= "            reference=value.substring(type.length);"."\n";
        $result.= "            txt='";
        $result.= '<div id="blockmultimovie-container-' . $this->instance->id . '">';
        $result.= '<object type="application/x-shockwave-flash" ';
        $result.= 'data="http://www.youtube.com/v/'."'+reference+'".'&amp;autoplay=1" ';
        $result.= 'width="170" height="140">';
        $result.= '<param name="movie" value="http://www.youtube.com/v/'."'+reference+'".'&amp;autoplay=1"><\/param>';
        $result.= '<param name="wmode" value="transparent"><\/param>';
        $result.= '<\/object>';
        $result.= '<\/div>'."';"."\n";
        $result.= "        } else if (value.indexOf('google') != -1) {"."\n";
        $result.= "            type='google';"."\n";
        $result.= "            reference=value.substring(type.length);"."\n";
        $result.= "            txt='";
        $result.= '<div id="blockmultimovie-container-' . $this->instance->id . '">';
        $result.= '<object type="application/x-shockwave-flash" ';
        $result.= 'data="http://video.google.com/googleplayer.swf?docId='."'+reference+'".'&amp;hl=en&amp;playerMode=simple&amp;autoPlay=true" ';
        $result.= 'width="170" height="140">';
        $result.= '<param name="movie" value="http://video.google.com/googleplayer.swf?docId='."'+reference+'".'&amp;hl=en&amp;playerMode=simple&amp;autoPlay=true"><\/param>';
        $result.= '<param name="wmode" value="transparent"><\/param>';
        $result.= '<\/object>';
        $result.= '<\/div>'."';"."\n";
        $result.= "        } else if (value.indexOf('teachertube') != -1) {"."\n";
        $result.= "            type='teachertube';"."\n";
        $result.= "            reference=value.substring(type.length);"."\n";
        $result.= "            txt='";
        $result.= '<div id="blockmultimovie-container-' . $this->instance->id . '">';
        $result.= '<object type="application/x-shockwave-flash" ';
        $result.= 'data="http://www.teachertube.com/flvplayer.swf" ';
        $result.= 'width="170" height="140">';
        $result.= '<param name="FlashVars" value="config=http://www.teachertube.com/flvplayer.php?viewkey='."'+reference+'".'"><\/param>';
        $result.= '<param name="wmode" value="transparent"><\/param>';
        $result.= '<\/object>';
        $result.= '<\/div>'."';"."\n";
        $result.= "        }"."\n";
    /// dinamic content, based on type
    /// find block div
        $result.= '        var thediv=document.getElementById('."'";
        $result.= 'blockmultimovie-' . $this->instance->id;
        $result.= "'". ');'."\n";
        $result.= '        thediv.innerHTML=txt;'."\n";

    /// script footer
        $result.= '    }'."\n";
        $result.= '//]]>'."\n";
        $result.= '</script>'."\n";

    /// return results
        return $result;
    }
}
?>
